@@ -1,3 +1,6 @@
+import tblib.pickling_support
+tblib.pickling_support.install()
+
 import abc
 import collections
 import dill
@@ -6,14 +9,26 @@ import functools
 import itertools
 import multiprocessing
 import os
+import sys
 import Queue
 import random
 import threading
 import time
+import traceback
 
 from pacer.std_logger import get_logger
 
 
+class DelayedException(object):
+
+    def __init__(self, ee):
+        self.ee = ee
+        __,  __, self.tb = sys.exc_info()
+
+    def re_raise(self):
+        raise self.ee, None, self.tb
+
+
 def remote_eval(f_s, args_s):
     """ requires function and args serialized with dill, so we can remote execute even lambdas or
     decorated functions or static methods, which 'pure' multiprocssing can not handle"""
@@ -21,12 +36,12 @@ def remote_eval(f_s, args_s):
     try:
         args = dill.loads(args_s)
     except Exception, e:
-        get_logger(remote_eval).error("got exception when unpicling args: {}".format(e))
+        get_logger(remote_eval).error("got exception when unpickling args: {}".format(e))
         return Exception(e.message)
     try:
         f = dill.loads(f_s)
     except Exception, e:
-        get_logger(remote_eval).error("got exception when unpicling f: {}".format(e))
+        get_logger(remote_eval).error("got exception when unpickling f: {}".format(e))
         return Exception(e.message)
     try:
         get_logger(f).info("start {}{} in process {}".format(f.__name__, args, os.getpid()))
@@ -35,8 +50,7 @@ def remote_eval(f_s, args_s):
         return result
     except Exception, e:
         get_logger(f).error("got exception: {}".format(e))
-        args_s = ", ".join("{}".format(ai) for ai in args)
-        return Exception(e.message + ", source: {}({})".format(f.__name__, args_s))
+        return DelayedException(e)
 
 
 class EnumeratedItem(object):
@@ -93,7 +107,10 @@ class Engine(object):
         assert all(isinstance(arg, EnumeratedItem) for arg in enumerated_args)
         new_enumeration, args = unpack(enumerated_args)
         if Engine.pool is None:
-            wrap_callback(new_enumeration, callback)(f(*args))
+            f_s = dill.dumps(f)
+            args_s = dill.dumps(args)
+            result = remote_eval(f_s, args_s)
+            wrap_callback(new_enumeration, callback)(result)
         else:
             # dill can pickle more function types as pickle module (which underlies
             # multiprocessing) , eg lambdas or static methods, so we transport our computation job
@@ -120,8 +137,13 @@ class DataStream(object):
     def size(self):
         return self._size
 
-    def put(self, value):
-        self._stream.put(value)
+    def put(self, item):
+        """this is where computation results are passed to."""
+        if isinstance(item.value, DelayedException):
+            item.value.re_raise()
+            item.value.print_tb()
+            raise Exception()
+        self._stream.put(item)
 
     def get_all_in_order(self):
         results = [self.get() for _ in range(self.size())]
@@ -234,7 +256,6 @@ class JoinComputation(Computation):
 
 class SummarizeComputation(Computation):
 
-
     def __init__(self, f, inputs):
         assert len(inputs) == 1, "reduce computations only accept one input stream"
         super(SummarizeComputation, self).__init__(f, inputs)
@@ -318,6 +339,7 @@ class ZipComputation(Computation):
                 else:
                     time.sleep(0.001)
 
+
 def apply(inner):
     @functools.wraps(inner)
     def wrapped(*args):
